package javax.management.relation;

import javax.management.InstanceNotFoundException;
import javax.management.ReflectionException;
import javax.management.MBeanException;
import java.util.Iterator;
import java.util.ArrayList;
import com.sun.jmx.mbeanserver.Util;
import java.util.List;
import com.sun.jmx.defaults.JmxProperties;
import java.util.HashMap;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.Map;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.MBeanRegistration;

public class RelationSupport implements RelationSupportMBean, MBeanRegistration
{
    private String myRelId;
    private ObjectName myRelServiceName;
    private MBeanServer myRelServiceMBeanServer;
    private String myRelTypeName;
    private final Map<String, Role> myRoleName2ValueMap;
    private final AtomicBoolean myInRelServFlg;
    
    public RelationSupport(String s, ObjectName objectName, String s2, RoleList list) throws InvalidRoleValueException, IllegalArgumentException {
        this.myRelId = null;
        this.myRelServiceName = null;
        this.myRelServiceMBeanServer = null;
        this.myRelTypeName = null;
        this.myRoleName2ValueMap = new HashMap();
        this.myInRelServFlg = new AtomicBoolean();
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "RelationSupport");
        this.initMembers(s, objectName, null, s2, list);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "RelationSupport");
    }
    
    public RelationSupport(String s, ObjectName objectName, MBeanServer mBeanServer, String s2, RoleList list) throws InvalidRoleValueException, IllegalArgumentException {
        this.myRelId = null;
        this.myRelServiceName = null;
        this.myRelServiceMBeanServer = null;
        this.myRelTypeName = null;
        this.myRoleName2ValueMap = new HashMap();
        this.myInRelServFlg = new AtomicBoolean();
        if (mBeanServer == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "RelationSupport");
        this.initMembers(s, objectName, mBeanServer, s2, list);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "RelationSupport");
        return;
    }
    
    @Override
    public List<ObjectName> getRole(String param1) throws IllegalArgumentException, RoleNotFoundException, RelationServiceNotRegisteredException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getRole", param1);
        List<Object> list = (List)Util.cast(this.getRoleInt(param1, false, null, false));
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getRole");
        return list;
    }
    
    @Override
    public RoleResult getRoles(String[] array) throws IllegalArgumentException, RelationServiceNotRegisteredException {
        if (array == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getRoles");
        RoleResult rolesInt = this.getRolesInt(array, false, null);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getRoles");
        return rolesInt;
    }
    
    @Override
    public RoleResult getAllRoles() throws RelationServiceNotRegisteredException {
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getAllRoles");
        RoleResult allRolesInt = null;
        try {
            allRolesInt = this.getAllRolesInt(false, null);
        }
        catch (IllegalArgumentException ex) {}
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getAllRoles");
        return allRolesInt;
    }
    
    @Override
    public RoleList retrieveAllRoles() {
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "retrieveAllRoles");
        RoleList list;
        synchronized (this.myRoleName2ValueMap) {
            list = new RoleList(new ArrayList(this.myRoleName2ValueMap.values()));
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "retrieveAllRoles");
        return list;
    }
    
    @Override
    public Integer getRoleCardinality(String param1) throws IllegalArgumentException, RoleNotFoundException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getRoleCardinality", param1);
        Role role;
        synchronized (this.myRoleName2ValueMap) {
            role = (Role)this.myRoleName2ValueMap.get(param1);
        }
        if (role == null) {
            int n = 1;
            try {
                RelationService.throwRoleProblemException(n, param1);
            }
            catch (InvalidRoleValueException ex) {}
        }
        List<ObjectName> roleValue = role.getRoleValue();
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getRoleCardinality");
        return Integer.valueOf(roleValue.size());
    }
    
    @Override
    public void setRole(Role param1) throws IllegalArgumentException, RoleNotFoundException, RelationTypeNotFoundException, InvalidRoleValueException, RelationServiceNotRegisteredException, RelationNotFoundException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "setRole", param1);
        this.setRoleInt(param1, false, null, false);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "setRole");
        return;
    }
    
    @Override
    public RoleResult setRoles(RoleList param1) throws IllegalArgumentException, RelationServiceNotRegisteredException, RelationTypeNotFoundException, RelationNotFoundException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "setRoles", param1);
        RoleResult setRolesInt = this.setRolesInt(param1, false, null);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "setRoles");
        return setRolesInt;
    }
    
    @Override
    public void handleMBeanUnregistration(ObjectName objectName, String s) throws IllegalArgumentException, RoleNotFoundException, InvalidRoleValueException, RelationServiceNotRegisteredException, RelationTypeNotFoundException, RelationNotFoundException {
        if (objectName == null || s == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "handleMBeanUnregistration", new Object[] { objectName, s });
        this.handleMBeanUnregistrationInt(objectName, s, false, null);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "handleMBeanUnregistration");
        return;
    }
    
    @Override
    public Map<ObjectName, List<String>> getReferencedMBeans() {
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getReferencedMBeans");
        HashMap<ObjectName, List> hashMap = new HashMap();
        synchronized (this.myRoleName2ValueMap) {
            for (Role role : this.myRoleName2ValueMap.values()) {
                String roleName = role.getRoleName();
                for (ObjectName objectName : role.getRoleValue()) {
                    List<String> list = (List)hashMap.get(objectName);
                    boolean n = false;
                    if (list == null) {
                        n = true;
                        list = new ArrayList();
                    }
                    list.add(roleName);
                    if (!(n == 0)) {
                        hashMap.put(objectName, list);
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getReferencedMBeans");
        return hashMap;
    }
    
    @Override
    public String getRelationTypeName() {
        return this.myRelTypeName;
    }
    
    @Override
    public ObjectName getRelationServiceName() {
        return this.myRelServiceName;
    }
    
    @Override
    public String getRelationId() {
        return this.myRelId;
    }
    
    @Override
    public ObjectName preRegister(MBeanServer myRelServiceMBeanServer, ObjectName objectName) throws Exception {
        this.myRelServiceMBeanServer = myRelServiceMBeanServer;
        return objectName;
    }
    
    @Override
    public void postRegister(Boolean b) {
    }
    
    @Override
    public void preDeregister() throws Exception {
    }
    
    @Override
    public void postDeregister() {
    }
    
    @Override
    public Boolean isInRelationService() {
        return Boolean.valueOf(this.myInRelServFlg.get());
    }
    
    @Override
    public void setRelationServiceManagementFlag(Boolean b) throws IllegalArgumentException {
        if (b == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        this.myInRelServFlg.set((boolean)b);
        return;
    }
    
    Object getRoleInt(String param1, boolean b, RelationService relationService, boolean b2) throws IllegalArgumentException, RoleNotFoundException, RelationServiceNotRegisteredException {
        if (param1 == null || (b && relationService == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getRoleInt", param1);
        Role role;
        synchronized (this.myRoleName2ValueMap) {
            role = (Role)this.myRoleName2ValueMap.get(param1);
        }
        int intValue;
        if (role == null) {
            intValue = 1;
        }
        else {
            Integer checkRoleReading;
            if (b) {
                try {
                    checkRoleReading = relationService.checkRoleReading(param1, this.myRelTypeName);
                }
                catch (RelationTypeNotFoundException ex4) {
                    throw new RuntimeException(ex4.getMessage());
                }
            }
            else {
                Object[] array = { param1, this.myRelTypeName };
                String[] array2 = { "java.lang.String", "java.lang.String" };
                try {
                    checkRoleReading = (Integer)(Integer)this.myRelServiceMBeanServer.invoke(this.myRelServiceName, "checkRoleReading", array, array2);
                }
                catch (MBeanException ex) {
                    throw new RuntimeException("incorrect relation type");
                }
                catch (ReflectionException ex2) {
                    throw new RuntimeException(ex2.getMessage());
                }
                catch (InstanceNotFoundException ex3) {
                    throw new RelationServiceNotRegisteredException(ex3.getMessage());
                }
            }
            intValue = (int)checkRoleReading;
        }
        Object o;
        if (intValue == 0) {
            if (!b2) {
                o = new ArrayList(role.getRoleValue());
            }
            else {
                o = (Role)(Role)role.clone();
            }
        }
        else if (!b2) {
            try {
                RelationService.throwRoleProblemException(intValue, param1);
                return null;
            }
            catch (InvalidRoleValueException ex5) {
                throw new RuntimeException(ex5.getMessage());
            }
        }
        else {
            o = new RoleUnresolved(param1, null, intValue);
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getRoleInt");
        return o;
    }
    
    RoleResult getRolesInt(String[] array, boolean b, RelationService relationService) throws IllegalArgumentException, RelationServiceNotRegisteredException {
        if (array == null || (b && relationService == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getRolesInt");
        RoleList list = new RoleList();
        RoleUnresolvedList list2 = new RoleUnresolvedList();
        for (int i = 0; i < array.length; ++i) {
            String s = array[i];
            Object roleInt;
            try {
                roleInt = this.getRoleInt(s, b, relationService, true);
            }
            catch (RoleNotFoundException ex) {
                return null;
            }
            if ((roleInt instanceof Role)) {
                try {
                    list.add((Role)roleInt);
                }
                catch (IllegalArgumentException ex3) {
                    throw new RuntimeException(ex3.getMessage());
                }
            }
            else if ((roleInt instanceof RoleUnresolved)) {
                try {
                    list2.add((RoleUnresolved)roleInt);
                }
                catch (IllegalArgumentException ex2) {
                    throw new RuntimeException(ex2.getMessage());
                }
            }
        }
        RoleResult o = new RoleResult(list, list2);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getRolesInt");
        return o;
    }
    
    RoleResult getAllRolesInt(boolean b, RelationService relationService) throws IllegalArgumentException, RelationServiceNotRegisteredException {
        if (b && relationService == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "getAllRolesInt");
        ArrayList list;
        synchronized (this.myRoleName2ValueMap) {
            list = new ArrayList(this.myRoleName2ValueMap.keySet());
        }
        String[] array = new String[list.size()];
        list.toArray(array);
        RoleResult rolesInt = this.getRolesInt(array, b, relationService);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "getAllRolesInt");
        return rolesInt;
    }
    
    Object setRoleInt(Role role, boolean b, RelationService relationService, boolean b2) throws IllegalArgumentException, RoleNotFoundException, InvalidRoleValueException, RelationServiceNotRegisteredException, RelationTypeNotFoundException, RelationNotFoundException {
        if (role == null || (b && relationService == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "setRoleInt", new Object[] { role, Boolean.valueOf(b), relationService, Boolean.valueOf(b2) });
        String roleName = role.getRoleName();
        Role role2;
        synchronized (this.myRoleName2ValueMap) {
            role2 = (Role)this.myRoleName2ValueMap.get(roleName);
        }
        Boolean b3;
        ArrayList<ObjectName> roleValue;
        if (role2 == null) {
            b3 = Boolean.valueOf(true);
            roleValue = new ArrayList();
        }
        else {
            b3 = Boolean.valueOf(false);
            roleValue = role2.getRoleValue();
        }
        int intValue;
        try {
            Integer checkRoleWriting;
            if (b) {
                checkRoleWriting = relationService.checkRoleWriting(role, this.myRelTypeName, b3);
            }
            else {
                checkRoleWriting = (Integer)(Integer)this.myRelServiceMBeanServer.invoke(this.myRelServiceName, "checkRoleWriting", new Object[] { role, this.myRelTypeName, b3 }, new String[] { "javax.management.relation.Role", "java.lang.String", "java.lang.Boolean" });
            }
            intValue = (int)checkRoleWriting;
        }
        catch (MBeanException ex) {
            Exception targetException = ex.getTargetException();
            if ((targetException instanceof RelationTypeNotFoundException)) {
                throw (RelationTypeNotFoundException)targetException;
            }
            throw new RuntimeException(targetException.getMessage());
        }
        catch (ReflectionException ex2) {
            throw new RuntimeException(ex2.getMessage());
        }
        catch (RelationTypeNotFoundException ex3) {
            throw new RuntimeException(ex3.getMessage());
        }
        catch (InstanceNotFoundException ex4) {
            throw new RelationServiceNotRegisteredException(ex4.getMessage());
        }
        Object o = null;
        if (intValue == 0) {
            if (!(boolean)b3) {
                this.sendRoleUpdateNotification(role, roleValue, b, relationService);
                this.updateRelationServiceMap(role, roleValue, b, relationService);
            }
            synchronized (this.myRoleName2ValueMap) {
                this.myRoleName2ValueMap.put(roleName, (Role)(Role)role.clone());
            }
            if (b2) {
                o = role;
            }
        }
        else {
            if (!b2) {
                RelationService.throwRoleProblemException(intValue, roleName);
                return null;
            }
            o = new RoleUnresolved(roleName, role.getRoleValue(), intValue);
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "setRoleInt");
        return o;
    }
    
    private void sendRoleUpdateNotification(Role role, List<ObjectName> list, boolean b, RelationService relationService) throws IllegalArgumentException, RelationServiceNotRegisteredException, RelationNotFoundException {
        if (role == null || list == null || (b && relationService == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "sendRoleUpdateNotification", new Object[] { role, list, Boolean.valueOf(b), relationService });
        if (b) {
            try {
                relationService.sendRoleUpdateNotification(this.myRelId, role, list);
            }
            catch (RelationNotFoundException ex4) {
                throw new RuntimeException(ex4.getMessage());
            }
        }
        else {
            Object[] array = { this.myRelId, role, list };
            String[] array2 = { "java.lang.String", "javax.management.relation.Role", "java.util.List" };
            try {
                this.myRelServiceMBeanServer.invoke(this.myRelServiceName, "sendRoleUpdateNotification", array, array2);
            }
            catch (ReflectionException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            catch (InstanceNotFoundException ex2) {
                throw new RelationServiceNotRegisteredException(ex2.getMessage());
            }
            catch (MBeanException ex3) {
                Exception targetException = ex3.getTargetException();
                if ((targetException instanceof RelationNotFoundException)) {
                    throw (RelationNotFoundException)targetException;
                }
                throw new RuntimeException(targetException.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "sendRoleUpdateNotification");
        return;
    }
    
    private void updateRelationServiceMap(Role role, List<ObjectName> list, boolean b, RelationService relationService) throws IllegalArgumentException, RelationServiceNotRegisteredException, RelationNotFoundException {
        if (role == null || list == null || (b && relationService == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "updateRelationServiceMap", new Object[] { role, list, Boolean.valueOf(b), relationService });
        if (b) {
            try {
                relationService.updateRoleMap(this.myRelId, role, list);
            }
            catch (RelationNotFoundException ex4) {
                throw new RuntimeException(ex4.getMessage());
            }
        }
        else {
            Object[] array = { this.myRelId, role, list };
            String[] array2 = { "java.lang.String", "javax.management.relation.Role", "java.util.List" };
            try {
                this.myRelServiceMBeanServer.invoke(this.myRelServiceName, "updateRoleMap", array, array2);
            }
            catch (ReflectionException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            catch (InstanceNotFoundException ex2) {
                throw new RelationServiceNotRegisteredException(ex2.getMessage());
            }
            catch (MBeanException ex3) {
                Exception targetException = ex3.getTargetException();
                if ((targetException instanceof RelationNotFoundException)) {
                    throw (RelationNotFoundException)targetException;
                }
                throw new RuntimeException(targetException.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "updateRelationServiceMap");
        return;
    }
    
    RoleResult setRolesInt(RoleList list, boolean b, RelationService relationService) throws IllegalArgumentException, RelationServiceNotRegisteredException, RelationTypeNotFoundException, RelationNotFoundException {
        if (list == null || (b && relationService == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "setRolesInt", new Object[] { list, Boolean.valueOf(b), relationService });
        RoleList list2 = new RoleList();
        RoleUnresolvedList list3 = new RoleUnresolvedList();
        for (Role role : list.asList()) {
            Object setRoleInt = null;
            try {
                setRoleInt = this.setRoleInt(role, b, relationService, true);
            }
            catch (RoleNotFoundException ex) {}
            catch (InvalidRoleValueException ex2) {}
            if ((setRoleInt instanceof Role)) {
                try {
                    list2.add((Role)setRoleInt);
                }
                catch (IllegalArgumentException ex4) {
                    throw new RuntimeException(ex4.getMessage());
                }
            }
            else if ((setRoleInt instanceof RoleUnresolved)) {
                try {
                    list3.add((RoleUnresolved)setRoleInt);
                }
                catch (IllegalArgumentException ex3) {
                    throw new RuntimeException(ex3.getMessage());
                }
            }
            else {
                continue;
            }
        }
        RoleResult o = new RoleResult(list2, list3);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "setRolesInt");
        return o;
    }
    
    private void initMembers(String myRelId, ObjectName myRelServiceName, MBeanServer myRelServiceMBeanServer, String myRelTypeName, RoleList list) throws InvalidRoleValueException, IllegalArgumentException {
        if (myRelId == null || myRelServiceName == null || myRelTypeName == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "initMembers", new Object[] { myRelId, myRelServiceName, myRelServiceMBeanServer, myRelTypeName, list });
        this.myRelId = myRelId;
        this.myRelServiceName = myRelServiceName;
        this.myRelServiceMBeanServer = myRelServiceMBeanServer;
        this.myRelTypeName = myRelTypeName;
        this.initRoleMap(list);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "initMembers");
        return;
    }
    
    private void initRoleMap(RoleList param1) throws InvalidRoleValueException {
        if (param1 == null) {
            return;
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "initRoleMap", param1);
        synchronized (this.myRoleName2ValueMap) {
            for (Role role : param1.asList()) {
                String roleName = role.getRoleName();
                if (this.myRoleName2ValueMap.containsKey(roleName)) {
                    StringBuilder sb = new StringBuilder("Role name ");
                    sb.append(roleName);
                    sb.append(" used for two roles.");
                    throw new InvalidRoleValueException(sb.toString());
                }
                this.myRoleName2ValueMap.put(roleName, (Role)(Role)role.clone());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "initRoleMap");
        return;
    }
    
    void handleMBeanUnregistrationInt(ObjectName objectName, String str, boolean b, RelationService relationService) throws IllegalArgumentException, RoleNotFoundException, InvalidRoleValueException, RelationServiceNotRegisteredException, RelationTypeNotFoundException, RelationNotFoundException {
        if (objectName == null || str == null || (b && relationService == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationSupport.class.getName(), "handleMBeanUnregistrationInt", new Object[] { objectName, str, Boolean.valueOf(b), relationService });
        Role role;
        synchronized (this.myRoleName2ValueMap) {
            role = (Role)this.myRoleName2ValueMap.get(str);
        }
        if (role == null) {
            StringBuilder sb = new StringBuilder();
            sb.append("No role with name ");
            sb.append(str);
            throw new RoleNotFoundException(sb.toString());
        }
        ArrayList<Object> list = new ArrayList(role.getRoleValue());
        list.remove(objectName);
        this.setRoleInt(new Role(str, list), b, relationService, false);
        JmxProperties.RELATION_LOGGER.exiting(RelationSupport.class.getName(), "handleMBeanUnregistrationInt");
        return;
    }
}
